@@ -4,6 +4,7 @@
 
     :license: MIT, see LICENSE for more details.
 """
+from SoftLayer import exceptions
 from SoftLayer import testing
 
 import json
@@ -28,10 +29,37 @@ class BlockTests(testing.TestCase):
         self.assert_called_with('SoftLayer_Billing_Item', 'cancelItem',
                                 args=(False, True, None))
 
+    def test_volume_set_lun_id_in_range(self):
+        lun_mock = self.set_mock('SoftLayer_Network_Storage', 'createOrUpdateLunId')
+        lun_mock.return_value = dict(volumeId=1234, value='42')
+        result = self.run_command('block volume-set-lun-id 1234 42'.split())
+        self.assert_no_fail(result)
+        self.assertEqual('Block volume with id 1234 is reporting LUN ID 42\n',
+                         result.output)
+
+    def test_volume_set_lun_id_in_range_missing_value(self):
+        lun_mock = self.set_mock('SoftLayer_Network_Storage', 'createOrUpdateLunId')
+        lun_mock.return_value = dict(volumeId=1234)
+        result = self.run_command('block volume-set-lun-id 1234 42'.split())
+        self.assert_no_fail(result)
+        self.assertEqual('Failed to confirm the new LUN ID on volume 1234\n',
+                         result.output)
+
+    def test_volume_set_lun_id_not_in_range(self):
+        value = '-1'
+        lun_mock = self.set_mock('SoftLayer_Network_Storage', 'createOrUpdateLunId')
+        lun_mock.side_effect = exceptions.SoftLayerAPIError(
+            'SoftLayer_Exception_Network_Storage_Iscsi_InvalidLunId',
+            'The LUN ID specified is out of the valid range: %s [min: 0 max: 4095]' % (value))
+        result = self.run_command('block volume-set-lun-id 1234 42'.split())
+        self.assertIsNotNone(result.exception)
+        self.assertIn('The LUN ID specified is out of the valid range', result.exception.faultString)
+
     def test_volume_detail(self):
         result = self.run_command(['block', 'volume-detail', '1234'])
 
         self.assert_no_fail(result)
+        isinstance(json.loads(result.output)['IOPs'], float)
         self.assertEqual({
             'Username': 'username',
             'LUN Id': '2',
@@ -44,7 +72,8 @@ class BlockTests(testing.TestCase):
             'Data Center': 'dal05',
             'Type': 'ENDURANCE',
             'ID': 100,
-            '# of Active Transactions': '0',
+            '# of Active Transactions': '1',
+            'Ongoing Transaction': 'This is a buffer time in which the customer may cancel the server',
             'Replicant Count': '1',
             'Replication Status': 'Replicant Volume Provisioning '
                                   'has completed.',
@@ -59,11 +88,13 @@ class BlockTests(testing.TestCase):
                 {'Replicant ID': 'Data Center', '1785': 'dal01'},
                 {'Replicant ID': 'Schedule', '1785': 'REPLICATION_DAILY'},
             ]],
-            'Duplicate Volume Properties': [
-                {'Original Volume Name': 'Original Volume Size',
-                 'test-origin-volume-name': '20'},
-                {'Original Volume Name': 'Original Snapshot Name',
-                 'test-origin-volume-name': 'test-origin-snapshot-name'}
+            'Original Volume Properties': [
+                {'Property': 'Original Volume Size',
+                 'Value': '20'},
+                {'Property': 'Original Volume Name',
+                 'Value': 'test-original-volume-name'},
+                {'Property': 'Original Snapshot Name',
+                 'Value': 'test-original-snapshot-name'}
             ]
         }, json.loads(result.output))
 
@@ -75,16 +106,36 @@ class BlockTests(testing.TestCase):
             {
                 'bytes_used': None,
                 'capacity_gb': 20,
-                'datacenter': None,
+                'datacenter': 'dal05',
                 'id': 100,
+                'iops': None,
                 'ip_addr': '10.1.2.3',
                 'lunId': None,
+                'rep_partner_count': None,
                 'storage_type': 'ENDURANCE',
                 'username': 'username',
                 'active_transactions': None
             }],
             json.loads(result.output))
 
+    @mock.patch('SoftLayer.BlockStorageManager.list_block_volumes')
+    def test_volume_count(self, list_mock):
+        list_mock.return_value = [
+            {'serviceResource': {'datacenter': {'name': 'dal05'}}},
+            {'serviceResource': {'datacenter': {'name': 'ams01'}}},
+            {'serviceResource': {'datacenter': {'name': 'dal05'}}}
+        ]
+
+        result = self.run_command(['block', 'volume-count'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(
+            {
+                'dal05': 2,
+                'ams01': 1
+            },
+            json.loads(result.output))
+
     def test_volume_order_performance_iops_not_given(self):
         result = self.run_command(['block', 'volume-order',
                                    '--storage-type=performance', '--size=20',
@@ -92,27 +143,12 @@ class BlockTests(testing.TestCase):
 
         self.assertEqual(2, result.exit_code)
 
-    def test_volume_order_performance_iops_out_of_range(self):
-        result = self.run_command(['block', 'volume-order',
-                                   '--storage-type=performance', '--size=20',
-                                   '--iops=80000', '--os-type=linux',
-                                   '--location=dal05'])
-
-        self.assertEqual(2, result.exit_code)
-
-    def test_volume_order_performance_iops_not_multiple_of_100(self):
-        result = self.run_command(['block', 'volume-order',
-                                   '--storage-type=performance', '--size=20',
-                                   '--iops=122', '--os-type=linux',
-                                   '--location=dal05'])
-
-        self.assertEqual(2, result.exit_code)
-
     def test_volume_order_performance_snapshot_error(self):
         result = self.run_command(['block', 'volume-order',
                                    '--storage-type=performance', '--size=20',
                                    '--iops=100', '--os-type=linux',
-                                   '--location=dal05', '--snapshot-size=10'])
+                                   '--location=dal05', '--snapshot-size=10',
+                                   '--service-offering=performance'])
 
         self.assertEqual(2, result.exit_code)
 
@@ -125,20 +161,22 @@ class BlockTests(testing.TestCase):
                     {'description': 'Performance Storage'},
                     {'description': 'Block Storage'},
                     {'description': '0.25 IOPS per GB'},
-                    {'description': '20 GB Storage Space'}]
+                    {'description': '20 GB Storage Space'},
+                    {'description': '10 GB Storage Space (Snapshot Space)'}]
             }
         }
 
         result = self.run_command(['block', 'volume-order',
                                    '--storage-type=performance', '--size=20',
                                    '--iops=100', '--os-type=linux',
-                                   '--location=dal05'])
+                                   '--location=dal05', '--snapshot-size=10'])
 
         self.assert_no_fail(result)
         self.assertEqual(result.output,
                          'Order #478 placed successfully!\n'
                          ' > Performance Storage\n > Block Storage\n'
-                         ' > 0.25 IOPS per GB\n > 20 GB Storage Space\n')
+                         ' > 0.25 IOPS per GB\n > 20 GB Storage Space\n'
+                         ' > 10 GB Storage Space (Snapshot Space)\n')
 
     def test_volume_order_endurance_tier_not_given(self):
         result = self.run_command(['block', 'volume-order',
@@ -158,7 +196,7 @@ class BlockTests(testing.TestCase):
                     {'description': '0.25 IOPS per GB'},
                     {'description': '20 GB Storage Space'},
                     {'description': '10 GB Storage Space (Snapshot Space)'}]
-                }
+            }
         }
 
         result = self.run_command(['block', 'volume-order',
@@ -187,6 +225,41 @@ class BlockTests(testing.TestCase):
                          'Order could not be placed! Please verify '
                          'your options and try again.\n')
 
+    def test_volume_order_hourly_billing_not_available(self):
+        result = self.run_command(['block', 'volume-order',
+                                   '--storage-type=endurance', '--size=20',
+                                   '--tier=0.25', '--os-type=linux',
+                                   '--location=dal10', '--billing=hourly',
+                                   '--service-offering=enterprise'])
+
+        self.assertEqual(2, result.exit_code)
+
+    @mock.patch('SoftLayer.BlockStorageManager.order_block_volume')
+    def test_volume_order_hourly_billing(self, order_mock):
+        order_mock.return_value = {
+            'placedOrder': {
+                'id': 10983647,
+                'items': [
+                    {'description': 'Storage as a Service'},
+                    {'description': 'Block Storage'},
+                    {'description': '20 GB Storage Space'},
+                    {'description': '200 IOPS'}]
+            }
+        }
+
+        result = self.run_command(['block', 'volume-order',
+                                   '--storage-type=endurance', '--size=20',
+                                   '--tier=0.25', '--os-type=linux',
+                                   '--location=dal10', '--billing=hourly',
+                                   '--service-offering=storage_as_a_service'])
+        self.assert_no_fail(result)
+        self.assertEqual(result.output,
+                         'Order #10983647 placed successfully!\n'
+                         ' > Storage as a Service\n'
+                         ' > Block Storage\n'
+                         ' > 20 GB Storage Space\n'
+                         ' > 200 IOPS\n')
+
     @mock.patch('SoftLayer.BlockStorageManager.order_block_volume')
     def test_volume_order_performance_manager_error(self, order_mock):
         order_mock.side_effect = ValueError('failure!')
@@ -223,6 +296,43 @@ class BlockTests(testing.TestCase):
                                    '--schedule-type=HOURLY'])
         self.assert_no_fail(result)
 
+    def test_list_volume_schedules(self):
+        result = self.run_command([
+            'block', 'snapshot-schedule-list', '12345678'])
+        self.assert_no_fail(result)
+        self.assertEqual([
+            {
+                "week": None,
+                "maximum_snapshots": None,
+                "hour": None,
+                "day_of_week": None,
+                "day": None,
+                "replication": None,
+                "date_of_month": None,
+                "month_of_year": None,
+                "active": "",
+                "date_created": "",
+                "type": "WEEKLY",
+                "id": 978,
+                "minute": '30'
+            },
+            {
+                "week": None,
+                "maximum_snapshots": None,
+                "hour": None,
+                "day_of_week": None,
+                "day": None,
+                "replication": '*',
+                "date_of_month": None,
+                "month_of_year": None,
+                "active": "",
+                "date_created": "",
+                "type": "INTERVAL",
+                "id": 988,
+                "minute": '*'
+            }
+        ], json.loads(result.output))
+
     def test_create_snapshot(self):
         result = self.run_command(['block', 'snapshot-create', '12345678'])
 
@@ -270,6 +380,28 @@ class BlockTests(testing.TestCase):
         self.assertEqual(result.output, 'Block volume 12345678 is being'
                          ' restored using snapshot 87654321\n')
 
+    @mock.patch('SoftLayer.BlockStorageManager.order_snapshot_space')
+    def test_snapshot_order_order_not_placed(self, order_mock):
+        order_mock.return_value = {}
+
+        result = self.run_command(['block', 'snapshot-order', '1234',
+                                   '--capacity=10', '--tier=0.25'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(result.output,
+                         'Order could not be placed! Please verify '
+                         'your options and try again.\n')
+
+    @mock.patch('SoftLayer.BlockStorageManager.order_snapshot_space')
+    def test_snapshot_order_performance_manager_error(self, order_mock):
+        order_mock.side_effect = ValueError('failure!')
+
+        result = self.run_command(['block', 'snapshot-order', '1234',
+                                   '--capacity=10', '--tier=0.25'])
+
+        self.assertEqual(2, result.exit_code)
+        self.assertEqual('Argument Error: failure!', result.exception.message)
+
     @mock.patch('SoftLayer.BlockStorageManager.order_snapshot_space')
     def test_snapshot_order(self, order_mock):
         order_mock.return_value = {
@@ -278,7 +410,7 @@ class BlockTests(testing.TestCase):
                 'items': [{'description':
                            '10 GB Storage Space (Snapshot Space)'}],
                 'status': 'PENDING_APPROVAL',
-                }
+            }
         }
 
         result = self.run_command(['block', 'snapshot-order', '1234',
@@ -327,6 +459,7 @@ class BlockTests(testing.TestCase):
     def test_replication_locations_unsuccessful(self, locations_mock):
         locations_mock.return_value = False
         result = self.run_command(['block', 'replica-locations', '1234'])
+        self.assert_no_fail(result)
         self.assertEqual('No data centers compatible for replication.\n',
                          result.output)
 
@@ -475,6 +608,62 @@ class BlockTests(testing.TestCase):
                          'Order #24601 placed successfully!\n'
                          ' > Storage as a Service\n')
 
+    @mock.patch('SoftLayer.BlockStorageManager.order_duplicate_volume')
+    def test_duplicate_order_hourly_billing(self, order_mock):
+        order_mock.return_value = {
+            'placedOrder': {
+                'id': 24602,
+                'items': [{'description': 'Storage as a Service'}]
+            }
+        }
+
+        result = self.run_command(['block', 'volume-duplicate', '100',
+                                   '--origin-snapshot-id=470',
+                                   '--duplicate-size=250',
+                                   '--duplicate-tier=2', '--billing=hourly',
+                                   '--duplicate-snapshot-size=20'])
+
+        order_mock.assert_called_with('100', origin_snapshot_id=470,
+                                      duplicate_size=250, duplicate_iops=None,
+                                      duplicate_tier_level=2,
+                                      duplicate_snapshot_size=20,
+                                      hourly_billing_flag=True)
+        self.assert_no_fail(result)
+        self.assertEqual(result.output,
+                         'Order #24602 placed successfully!\n'
+                         ' > Storage as a Service\n')
+
+    @mock.patch('SoftLayer.BlockStorageManager.order_modified_volume')
+    def test_modify_order_exception_caught(self, order_mock):
+        order_mock.side_effect = ValueError('order attempt failed, noooo!')
+
+        result = self.run_command(['block', 'volume-modify', '102', '--new-size=1000'])
+
+        self.assertEqual(2, result.exit_code)
+        self.assertEqual('Argument Error: order attempt failed, noooo!', result.exception.message)
+
+    @mock.patch('SoftLayer.BlockStorageManager.order_modified_volume')
+    def test_modify_order_order_not_placed(self, order_mock):
+        order_mock.return_value = {}
+
+        result = self.run_command(['block', 'volume-modify', '102', '--new-iops=1400'])
+
+        self.assert_no_fail(result)
+        self.assertEqual('Order could not be placed! Please verify your options and try again.\n', result.output)
+
+    @mock.patch('SoftLayer.BlockStorageManager.order_modified_volume')
+    def test_modify_order(self, order_mock):
+        order_mock.return_value = {'placedOrder': {'id': 24602, 'items': [{'description': 'Storage as a Service'},
+                                                                          {'description': '1000 GBs'},
+                                                                          {'description': '4 IOPS per GB'}]}}
+
+        result = self.run_command(['block', 'volume-modify', '102', '--new-size=1000', '--new-tier=4'])
+
+        order_mock.assert_called_with('102', new_size=1000, new_iops=None, new_tier_level=4)
+        self.assert_no_fail(result)
+        self.assertEqual('Order #24602 placed successfully!\n > Storage as a Service\n > 1000 GBs\n > 4 IOPS per GB\n',
+                         result.output)
+
     def test_set_password(self):
         result = self.run_command(['block', 'access-password', '1234', '--password=AAAAA'])
         self.assert_no_fail(result)
